/*
TcpPort.h

Socket Communication made to look like Serial I/O functions
*/

#pragma once

#include "Xpert/Win32Fix.h"
#include "GenericPort.h"
#include "Socket.h"

#undef DECL
#if defined(NOT_XPERT)
#define DECL
#else
#if defined(_AFXDLL) || defined(XPERT_EXE)
#ifdef UTILS_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif
#else
#define DECL 
#endif
#endif

class DECL CTcpPort : public CGenericPort
{
public:
   CTcpPort(TSocket& aSocket);
   ~CTcpPort();

   int GetPortType() { return ptTCPIP; }; // 0=Unknown, 1=RS232, 2=TCPIP

   // open/close
   bool CloseComm();
   bool OpenComm();
   bool IsOpen();

   BOOL ResetWake() {return TRUE; };
   // data
   bool GetChar(UCHAR * uch);
   bool GetChar(TCHAR * wch);

   int GetStr(UCHAR * Str, int StrLen);
   int GetStr(TCHAR * Str, int StrLen);
   int Get(void* Buffer, int BufferLen);
   int Get(void* Buffer, int BufferLen, DWORD dwTimeout);

   bool Put(const void* Buffer, int BufferLen, DWORD dwTimeout);
   bool PutAndWait(const void* Buffer, int BufferLen, DWORD dwTimeout);
   bool Put(const void* Buffer, int BufferLen);
   bool PutAndWait(const void* Buffer, int BufferLen);
   bool PutChar(const UCHAR uch);
   bool PutChar(const TCHAR wch);

   bool PutStr(const UCHAR * Str, int StrLen);
   bool PutStr(TCHAR * Str, int StrLen);
   bool PutStr(CString Str);

   int  NumberBytesInputBuffer(); // returns number of bytes in input buffer
   bool KeyPressed();             // return true if a key is in rx buffer
   void FlushInput();             // empties rx buffer

   HANDLE GetHandle() {return (HANDLE) Socket.GetSocket(); };

   void WaitOnRx();                // waits (blocks) until a char comes in the serial port - flushes input first
   void WaitForTxEmpty();          // waits (blocks) until transmit buffer empty.
   void WaitForTxEmpty(TCHAR LastCh); // More accurate version of WaitForTxEmpty()

   BOOL SetCommState(LPDCB lpDCB)               { return FALSE;                                      };
   BOOL GetCommState(LPDCB lpDCB)               { return FALSE;                                      };
   BOOL SetCommMask(DWORD dwEvtMask);
   BOOL GetCommMask(LPDWORD dwEvtMask);
   BOOL EscapeCommFunction(DWORD dwFunc)        { return FALSE;                                      };
   BOOL SetCommBreak()                          { return FALSE;                                      };
   BOOL ClearCommBreak()                        { return FALSE;                                      };
   BOOL GetCommModemStatus(LPDWORD lpModemStat);
   BOOL WaitCommEvent(LPDWORD lpEvtMask);

protected:
   DWORD GetReadTimeout(int Len=1);
   DWORD GetWriteTimeout(int Len=1);

protected:
   TSocket& Socket;
   HANDLE hStopWaiting;
   DWORD EvtMask;
   DWORD EvtStatus;
   bool WasConnected;
   DWORD dwLastConnectCheck;
};
