// XpertAPI.h : along with XpertAPI.cpp and Win32Fix.h, helps manage differences between Xpert and 
// desktop platforms. 
//

#ifndef _XpertAPI_H_
#define _XpertAPI_H_


#ifdef _WIN32_WCE
#include <ntddndis.h>
#include <winioctl.h>
#include <Xpert/i2c.h>
#include <Xpert/io.h>
#else 
#include "Xpert/Win32Fix.h"
#endif // _WIN32_WCE


#undef DECL
#ifdef UTILS_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif


// Used by API_GetCPLDXtraRegister() function
enum
{
   XTRA_I2CPOWER       = 1<<0, // I2C POWER Line
   XTRA_VREF           = 1<<1,
   XTRA_RTC            = 1<<2,
   XTRA_CPLDMODE_MASK  = 1<<3,
   XTRA_CF_PWR_DOWN    = 1<<4,
   XTRA_USBH_WAKE      = 1<<5,
   XTRA_CLK            = 1<<6, // I2C CLK Line
   XTRA_DAT            = 1<<7  // I2C DATA Line
};

extern "C" BYTE DECL API_GetCPLDXtraRegister();
/*
   Returns the state of the CPLD Xtra Register. See the bit masks above.
*/


// Used by API_SelectUartMode() function
#define UART_MODE_RS232 (1)         // Normal RS232 signaling
#define UART_MODE_RS422 (2)         // RS422 signalling
#define UART_MODE_RS485 (3)         // RS485 signalling
#define UART_MODE_RS485_HALF (4)    // RS485 signalling (half duplex)
#define UART_MODE_RS232_AUTORTS (5) // RS232 with AUTO RTS handling

extern "C" bool API_SelectUartMode(HANDLE hPort, int iUartMode);


// Used by API_GetDevicePwr() function
// Used by API_SetDevicePwr() function
#ifdef _WIN32_WCE
// See enum PWRMASK in io.h. 
#define DEVICE_PWR_OFF     OFF      // D4
#define DEVICE_PWR_WAKE    WAKE     // D3
#define DEVICE_PWR_IDLE    IDLE     // D2
#define DEVICE_PWR_LOW     LOW      // D1
#define DEVICE_PWR_ON      ON       // D0
// See enum DEVMASK in io.h. 
#define DEVICE_ID_LCD      LCD
#define DEVICE_ID_USB      USB
#define DEVICE_ID_ETH      ETH
#define DEVICE_ID_USRLEDS  USRLEDS
#define DEVICE_ID_SDCARD   SDCARD
#define DEVICE_ID_COM1     COM1
#define DEVICE_ID_COM2     COM2
#define DEVICE_ID_COM3     COM3
#define DEVICE_ID_COM4     COM4
#define DEVICE_ID_COM5     COM5
#define DEVICE_ID_COM6     COM6
#define DEVICE_ID_COM7     COM7
#define DEVICE_ID_COM8     COM8
#define DEVICE_ID_COM9     COM9
#else
#define DEVICE_PWR_OFF     (4)
#define DEVICE_PWR_WAKE    (3)
#define DEVICE_PWR_IDLE    (2)
#define DEVICE_PWR_LOW     (1)
#define DEVICE_PWR_ON      (0)

#define DEVICE_ID_LCD      (1<<0)
#define DEVICE_ID_USB      (1<<1)
#define DEVICE_ID_ETH      (1<<2)
#define DEVICE_ID_USRLEDS  (1<<3)
#define DEVICE_ID_SDCARD   (1<<4)
#define DEVICE_ID_COM1     (1<<5)
#define DEVICE_ID_COM2     (1<<6)
#define DEVICE_ID_COM3     (1<<7)
#define DEVICE_ID_COM4     (1<<8)
#define DEVICE_ID_COM5     (1<<9)
#define DEVICE_ID_COM6     (1<<10)
#define DEVICE_ID_COM7     (1<<11)
#define DEVICE_ID_COM8     (1<<12)
#define DEVICE_ID_COM9     (1<<13)
#endif

extern "C" bool DECL API_GetDevicePwr(DWORD dwDevice, DWORD& dwPower);
extern "C" bool DECL API_SetDevicePwr(DWORD dwDevices, DWORD dwPower);

extern "C" bool DECL API_SetContrast(int iSetting);

extern "C" int DECL API_GetContrast();

extern "C" bool DECL API_SetContrastToDefault();

extern "C" bool DECL API_GetBattAndTemp(double& dBattVolt, double& dTempC);

extern "C" int DECL API_ReadResetCount();

extern "C" void DECL API_WriteResetCount(int iResetCount);

extern "C" bool DECL API_IsXpert();

extern "C" void DECL API_Reboot();

// Shutdown the application and reboot - safer than calling API_Reboot directly - but this always returns
extern "C" void DECL API_Shutdown();

extern "C" HANDLE DECL API_OpenI2C(BYTE ClockFreq, BYTE MasterAddr);
/*
	Open's the I2C port
Passed:
	ClockFreq: Selects the bus frequency (from available options)
	MasterAddr: Sets the master address. Only messages addressed to the master are received.
Returns:
	A device handle or NULL if the port is inuse.
*/

extern "C" void DECL API_CloseI2C(HANDLE hI2C);
/*
	Closes the I2C port, waiting for any pending I/O operations to complete and terminating any that
	are blocked.
Passed:
	hI2C:     I2C Device Handle
*/

extern "C" int DECL API_ReadI2CMessage(HANDLE hI2C, void* pMessage, int MaxBytes, DWORD dwTimeoutMS);
/*
   Reads an I2C message from the port. A repeated start condition that occurs in the middle
   of the reception of a message should be handled automatically. The checksum of an incoming
   message needs to be computed after the checksum bytes are received, and if it is not correct the
   final two "stop bytes" must be nak'ed.
Passed:
	hI2C:        I2C Device Handle
	pMessage:    Pointer to a pre-allocated buffer that will contain an incoming message. 
	MaxBytes:    Number of bytes pMessage can hold
	dwTimeoutMS: How long to wait for a message before timing out
Returns:
	0: Operation successfull
	2: Timeout exceeded
	4: MaxBytes limit reached
	5: Bus Error (clock & data changed in an undefined manner or appear to be stuck)
	6: Rx Error (message stopped early, or some other error occured)
	7: Unexpected Slave TX revieved.
	8: Bad Checksum
	14: Handle is invalid
	15: Port was closed during operation
*/

extern "C" int DECL API_WriteI2CMessage(HANDLE hI2C, void* pMessage, DWORD dwTimeoutMS);
/*
	Transmits an I2C message. Retries (other then busbusy) are performed by higher-level code.
Passed:
	hI2C:        I2C Device Handle
	pMessage:    Pointer to a buffer contains the outgoing message
	dwTimeoutMS: How long to wait for the bus to be not busy before reporting an error
Returns:
	0: Operation successfull
	14: Handle is invalid
	15: Port was closed during operation
	1: Message was NAK'd
	10: Bus Busy Timeout
	3: Lost Arbitration
*/

extern "C" int DECL API_GetI2CBusState(HANDLE hI2C);
/*
   Returns the state of the I2C BUS signals
Passed:
	hI2C:        I2C Device Handle
Returns:
	bit 0: State of the I2C Clock Line
	bit 1: State of the I2C Data Line
	bit 2: State of the I2C BUS 5V Power
*/

extern "C" void DECL API_SetI2CPower(HANDLE hI2C, bool On);
/*
   Turns the +5V to BUS devices on or off
Passed:
	hI2C:        I2C Device Handle
	On:          true: Power on +5v to the bus; false: power off +5v to the bus.
*/

extern "C" bool DECL API_SetEthernetPower(int Option);
/*
   Turns the ethernet port on or off
   0=off, 1=on, 2=auto detect, 3=auto detect (once)
*/

extern "C" bool DECL API_SetUSBPower(int Option);
/*
   Turns the USB port on or off
   0=off, 1=on, 2=auto detect
*/


extern "C" DECL void API_GetSerialNum(CString& strSerialNum);
/* 
   Retrieves serial number of current device. 
*/

extern "C" DECL bool API_GetLoaderAndMonitorVersion(CString& strLoaderVersion, CString& strMonitorVersion);

/*
   -1: Not supportted (Xpert2)
   0 : First Revision 7210 CPLD
   etc
*/
extern "C" DECL int API_GetCPLDVersion();

// Returns the CPLD version in a string, ie: "7210, ver A"
extern DECL CString API_GetCPLDVersionStr();

#endif // _XpertAPI_H_
